#import "MSDelegateViewController.h"

@implementation MSDelegateViewController

- (id)initWithNibName:(NSString *)name bundle:(NSBundle *)bundle {
    if ((self = [super initWithNibName:name bundle:bundle])) {
        self.mapView.mapKey = [MREditorKey keyForMap:[MSExampleHost mapID] app:[MSExampleHost appID]];
    }
    return self;
}

#pragma mark - MSExampleViewController

+ (NSString *)exampleTitle {
    return @"Map Delegates";
}

+ (NSString *)exampleInfo {
    return @"Map that logs delegate callbacks.\n\n"
    @"• All MRMapViewDelegate methods are logged to console.\n";
}

#pragma mark - MRMapViewDelegate

- (void)mapPickerDidPickMap:(MRMap *)map {
    [super mapPickerDidPickMap:map];
    NSLog(@"%s", __PRETTY_FUNCTION__);
}

- (void)mapViewWillStartLoadingMap:(MRMapView *)mapView {
    [super mapViewWillStartLoadingMap:mapView];
    NSLog(@"%s", __FUNCTION__);
}

- (void)mapViewDidFinishLoadingMap:(MRMapView *)mapView {
    [super mapViewDidFinishLoadingMap:mapView];
    NSLog(@"%s", __FUNCTION__);
}

- (void)mapViewDidFailLoadingMap:(MRMapView *)mapView withError:(NSError *)error {
    [super mapViewDidFailLoadingMap:mapView withError:error];
    NSLog(@"%s - error:%@", __FUNCTION__, error);
}

- (void)mapViewDidChangeUseAccessiblePaths:(MRMapView *)mapView {
    [super mapViewDidChangeUseAccessiblePaths:mapView];
    NSLog(@"%s", __FUNCTION__);
}

- (void)mapView:(MRMapView *)mapView routeDidChange:(MRRoute *)route {
    [super mapView:mapView routeDidChange:route];
    NSLog(@"%s", __FUNCTION__);
}

- (void)mapView:(MRMapView *)mapView willScrollToStepAtIndex:(NSUInteger)index {
    [super mapView:mapView willScrollToStepAtIndex:index];
    NSLog(@"%s - index:%@", __FUNCTION__, @(index));
}

- (CGRect)mapView:(MRMapView *)mapView willScrollToRect:(CGRect)rect forRouteStep:(MRRouteStep *)step {
    CGRect superRect = [super mapView:mapView willScrollToRect:rect forRouteStep:step];
    NSLog(@"%s - rect:%@", __FUNCTION__, NSStringFromCGRect(rect));
    return superRect;
}

- (void)mapView:(MRMapView *)mapView visibleMapRectDidChange:(BOOL)animated {
    [super mapView:mapView visibleMapRectDidChange:animated];
    NSLog(@"%s - animated:%@", __FUNCTION__, @(animated));
}

- (MRAnnotationView *)mapView:(MRMapView *)mapView viewForAnnotation:(id<MRAnnotation>)annotation {
    MRAnnotationView *superAnnotationView = [super mapView:mapView viewForAnnotation:annotation];
    NSLog(@"%s - annotation:%@", __FUNCTION__, annotation.description);
    return superAnnotationView;
}

- (void)mapView:(MRMapView *)mapView didAddAnnotationViews:(NSArray *)views {
    [super mapView:mapView didAddAnnotationViews:views];
    NSLog(@"%s - views:%@", __FUNCTION__, @(views.count));
}

- (void)mapView:(MRMapView *)mapView didLoadPlacemarks:(NSArray *)placemarks {
    [super mapView:mapView didLoadPlacemarks:placemarks];
    NSLog(@"%s - placemarks:%@", __FUNCTION__, @(placemarks.count));
}

- (void)mapView:(MRMapView *)mapView didDeselectAnnotationView:(MRAnnotationView *)view {
    [super mapView:mapView didDeselectAnnotationView:view];
    NSLog(@"%s - view:%p", __FUNCTION__, view);
}

- (void)mapView:(MRMapView *)mapView didAddOverlayRenderers:(NSArray<MRPathRenderer *> *)renderers {
    [super mapView:mapView didAddOverlayRenderers:renderers];
    NSLog(@"%s - renderers:%@", __FUNCTION__, @(renderers.count));
}

- (void)mapView:(MRMapView *)mapView didLoadFriends:(NSArray<MRFriend *> *)friends {
    [super mapView:mapView didLoadFriends:friends];
    NSLog(@"%s - friends:%@", __FUNCTION__, @(friends.count));
}

- (void)mapView:(MRMapView *)mapView didLoadTags:(NSArray<MRTag *> *)tags {
    [super mapView:mapView didLoadTags:tags];
    NSLog(@"%s - tags:%@", __FUNCTION__, @(tags.count));
}

- (void)mapView:(MRMapView *)mapView didHighlightAnnotations:(nonnull NSArray<MRAnnotation> *)annotations {
    [super mapView:mapView didHighlightAnnotations:annotations];
    NSLog(@"%s - annotations:%@", __FUNCTION__, @(annotations.count));
}

- (void)mapView:(MRMapView *)mapView didUpdateTag:(MRTag *)tag {
    [super mapView:mapView didUpdateTag:tag];
    NSLog(@"%s - tag:%@", __FUNCTION__, tag.mac);
}

- (MRPathRenderer *)mapView:(MRMapView *)mapView rendererForOverlay:(MRPathOverlay *)overlay {
    MRPathRenderer *superPathRenderer = [super mapView:mapView rendererForOverlay:overlay];
    NSLog(@"%s", __FUNCTION__);
    return superPathRenderer;
}

- (void)mapView:(MRMapView *)mapView didSelectAnnotationView:(MRAnnotationView *)view {
    [super mapView:mapView didSelectAnnotationView:view];
    NSLog(@"%s - view:%p", __FUNCTION__, view);
}
- (void)mapView:(MRMapView *)mapView didUnhighlightAnnotations:(NSArray<MRAnnotation> *)annotations {
    [super mapView:mapView didHighlightAnnotations:annotations];
    NSLog(@"%s - annotations:%@", __FUNCTION__, @(annotations.count));
}

- (void)mapView:(MRMapView *)mapView didTapOverlay:(MRPathOverlay *)overlay {
    [super mapView:mapView didTapOverlay:overlay];
    NSLog(@"%s", __FUNCTION__);
}

- (MRDBottomSheetAbstractController *)viewControllerForMapView:(MRMapView *)mapView selectedAnnotationView:(MRAnnotationView *)view {
    MRDBottomSheetAbstractController *superViewController = [super viewControllerForMapView:mapView selectedAnnotationView:view];
    NSLog(@"%s - view:%p", __FUNCTION__, view);
    return superViewController;
}


- (MRAnnotationView *)mapView:(MRMapView *)mapView willSelectAnnotationView:(MRAnnotationView *)view {
    MRAnnotationView *superAnnotationView = [super mapView:mapView willSelectAnnotationView:view];
    NSLog(@"%s - view:%p", __FUNCTION__, view);
    return superAnnotationView;
}


- (MRPathOverlay *)mapView:(MRMapView *)mapView willTapOverlay:(MRPathOverlay *)overlay {
    MRPathOverlay *superPathOverlay = [super mapView:mapView willTapOverlay:overlay];
    NSLog(@"%s", __FUNCTION__);
    return superPathOverlay;
}

- (void)mapView:(MRMapView *)mapView locationProviderDidChangeFrom:(MRLocationProviderType)previousProviderType toProviderType:(MRLocationProviderType)newProviderType {
    [super mapView:mapView locationProviderDidChangeFrom:previousProviderType toProviderType:newProviderType];
    NSLog(@"%s - previous:%@ new:%@", __FUNCTION__, @(previousProviderType), @(newProviderType));
}

- (void)mapView:(MRMapView *)mapView didRequestMapPickerForMapKey:(MREditorKey *)key {
    [super mapView:mapView didRequestMapPickerForMapKey:key];
    NSLog(@"%s - key:%@", __FUNCTION__, key.identifier);
}

- (NSString *)mapPickerDetailStringForMap:(MRMap *)map {
    NSString *superDetailString = [super mapPickerDetailStringForMap:map];
    NSLog(@"%s - map:%@", __FUNCTION__, map.key.identifier);
    return superDetailString;
}


- (BOOL)shouldRespondToUserLocationButton { 
    BOOL superShouldRespond = [super shouldRespondToUserLocationButton];
    NSLog(@"%s", __FUNCTION__);
    return superShouldRespond;
}

- (void)mapView:(MRMapView *)mapView requestedDirectionsToPlacemark:(MRPlacemark *)placemark {
    [super mapView:mapView requestedDirectionsToPlacemark:placemark];
    NSLog(@"%s - placemark:%@", __FUNCTION__, placemark.key.identifier);
}

@end
