#import "MSInviteViewController.h"
#import "MSFriendManager.h"

@interface MSInviteViewController () <MSFriendManagerObserver>
@property (nonatomic, strong) MREditorKey *inviteKey;
@property (nonatomic, strong) MSFriendManager *manager;
@end

@implementation MSInviteViewController

- (id)initWithInvite:(MRInvite *)invite manager:(MSFriendManager *)manager {
    if ((self = [super initWithStyle:UITableViewStyleGrouped])) {
        self.inviteKey = invite.key;
        self.manager = manager;
        [self.manager addObserver:self];
    }
    return self;
}

- (void)dealloc {
    [self.manager removeObserver:self];
}

#pragma mark - UITableViewDataSource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return 2;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:nil];
    if (indexPath.row == 0) {
        cell.textLabel.text = @"Key";
        cell.detailTextLabel.text = self.invite.key.identifier;
    } else if (indexPath.row == 1) {
        cell.textLabel.text = @"Share URL";
        cell.detailTextLabel.text = self.invite.shareURL.absoluteString;
    }
    return cell;
}

#pragma mark - UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.row == 1) {
        UIActivityViewController *vc = [[UIActivityViewController alloc] initWithActivityItems:@[self.invite.shareURL] applicationActivities:nil];
        vc.popoverPresentationController.sourceView = tableView;

        [self presentViewController:vc animated:YES completion:NULL];
    }
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (BOOL)tableView:(UITableView *)tableView shouldShowMenuForRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}

- (BOOL)tableView:(UITableView *)tableView canPerformAction:(SEL)action forRowAtIndexPath:(NSIndexPath *)indexPath withSender:(id)sender {
    return (action == @selector(copy:));
}

- (void)tableView:(UITableView *)tableView performAction:(SEL)action forRowAtIndexPath:(NSIndexPath *)indexPath withSender:(id)sender {
    if (action == @selector(copy:)) {
        [UIPasteboard generalPasteboard].string = [self.tableView cellForRowAtIndexPath:indexPath].detailTextLabel.text ?: @"";
    }
}

#pragma mark - MSFriendManagerObserver

- (void)friendManagerDidUpdate:(MSFriendManager *)manager {
    [self.tableView reloadData];
}

- (void)friendManagerDidFailWithError:(NSError *)error {
    [self handleFriendError:error];
}

#pragma mark - Internal

- (MRInvite *)invite {
    for (MRInvite *i in self.manager.invites) {
        if ([i.key isEqual:self.inviteKey]) {
            return i;
        }
    }
    return nil;
}

@end
