#import "MSProfileViewController.h"
#import "MSFriendManager.h"

@interface MSProfileViewController () <UIImagePickerControllerDelegate, UINavigationControllerDelegate, MSFriendManagerObserver>
@property (nonatomic, strong) MSFriendManager *manager;
@property (nonatomic, strong) MRFriend *friend;
@end

@interface MRLocation (hush)
@property (nonatomic, strong) MREditorKey *mapKey;
@property (nonatomic, strong) NSDate *timestamp;
@property (nonatomic, assign) CGPoint point;
@end

@implementation MSProfileViewController

- (id)initWithManager:(MSFriendManager *)manager {
    if ((self = [super initWithStyle:UITableViewStyleGrouped])) {
        self.manager = manager;
        [self.manager addObserver:self];
    }
    return self;
}

- (id)initWithFriend:(MRFriend *)friend manager:(MSFriendManager *)manager {
    if ((self = [super initWithStyle:UITableViewStyleGrouped])) {
        self.friend = friend;
        self.manager = manager;
        [self.manager addObserver:self];
    }
    return self;
}

- (void)dealloc {
    [self.manager removeObserver:self];
}

#pragma mark - MSFriendManagerObserver

- (void)friendManagerDidUpdate:(MSFriendManager *)manager {
    [self.tableView reloadData];
}

- (void)friendManagerDidFailWithError:(NSError *)error {
    [self handleFriendError:error];
}

#pragma mark - UITableViewDataSource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return 9;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    MRFriend *profile = self.profile;
    UITableViewCell *cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:nil];
    if (indexPath.row == 0) {
        cell.textLabel.text = @"Key";
        cell.detailTextLabel.text = profile.key.identifier;
    } else if (indexPath.row == 1) {
        cell.textLabel.text = @"Name";
        cell.detailTextLabel.text = profile.name;
    } else if (indexPath.row == 2) {
        cell.textLabel.text = @"Profile Image";
        cell.detailTextLabel.text = profile.imageURL.absoluteString;
    } else if (indexPath.row == 3) {
        cell.textLabel.text = @"Sharing";
        cell.detailTextLabel.text = profile.sharing ? @"Yes" : @"No";
    } else if (indexPath.row == 4) {
        cell.textLabel.text = @"Map/App key";
        cell.detailTextLabel.text = profile.location.mapKey.description;
    } else if (indexPath.row == 5) {
        cell.textLabel.text = @"Point";
        cell.detailTextLabel.text = NSStringFromCGPoint(profile.location.point);
    } else if (indexPath.row == 6) {
        cell.textLabel.text = @"Timestamp";
        cell.detailTextLabel.text = profile.location.timestamp.description;
    } else if (indexPath.row == 7) {
        cell.textLabel.text = @"Distance";
        cell.detailTextLabel.text = profile.travelDistance.description;
    } else if (indexPath.row == 8) {
        cell.textLabel.text = @"Travel Time";
        cell.detailTextLabel.text = profile.travelTime.description;
    }
    return cell;
}

#pragma mark - UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    __weak typeof(self) weakSelf = self;
    if (indexPath.row == 1) {
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Update Name" message:nil preferredStyle:UIAlertControllerStyleAlert];
        [alert addTextFieldWithConfigurationHandler:^(UITextField *field){field.placeholder = @"Name";}];
        [alert addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action){
            [weakSelf.profile updateName:alert.textFields[0].text withSession:self.manager.session completion:^(MRFriend *friend, NSError *error){
                if (!error) {
                    [weakSelf.manager reloadData];
                } else {
                    [self handleFriendError:error];
                }
            }];
        }]];
        [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
        [self presentViewController:alert animated:YES completion:nil];
    } else if (indexPath.row == 2) {
        UIImagePickerController *picker = [[UIImagePickerController alloc] init];
        picker.delegate = self;
        picker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
        [self presentViewController:picker animated:YES completion:nil];
    } else if (indexPath.row == 3) {
        [self.profile disableSharingWithSession:self.manager.session completion:^(MRFriend *friend, NSError *error){
            if (!error) {
                [weakSelf.manager reloadData];
            } else {
                [self handleFriendError:error];
            }
        }];
    } else if (indexPath.row >= 4 || indexPath.row <= 5) {
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Update Location" message:nil preferredStyle:UIAlertControllerStyleAlert];
        [alert addTextFieldWithConfigurationHandler:^(UITextField *field){field.placeholder = @"App key";}];
        [alert addTextFieldWithConfigurationHandler:^(UITextField *field){field.placeholder = @"Map key";}];
        [alert addTextFieldWithConfigurationHandler:^(UITextField *field){field.placeholder = @"x";}];
        [alert addTextFieldWithConfigurationHandler:^(UITextField *field){field.placeholder = @"y";}];
        [alert addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action){
            MRLocation *location = [[MRLocation alloc] init];
            location.mapKey = [MREditorKey keyForMap:alert.textFields[1].text app:alert.textFields[0].text];
            location.point = CGPointMake(alert.textFields[2].text.floatValue, alert.textFields[3].text.floatValue);
            location.timestamp = [NSDate date];
            [weakSelf.profile updateLocation:location withSession:self.manager.session completion:^(MRFriend *friend, NSError *error){
                if (!error) {
                    [weakSelf.manager reloadData];
                } else {
                    [self handleFriendError:error];
                }
            }];
        }]];
        [alert addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil]];
        [self presentViewController:alert animated:YES completion:nil];
    }
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (BOOL)tableView:(UITableView *)tableView shouldShowMenuForRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}

- (BOOL)tableView:(UITableView *)tableView canPerformAction:(SEL)action forRowAtIndexPath:(NSIndexPath *)indexPath withSender:(id)sender {
    return (action == @selector(copy:));
}

- (void)tableView:(UITableView *)tableView performAction:(SEL)action forRowAtIndexPath:(NSIndexPath *)indexPath withSender:(id)sender {
    if (action == @selector(copy:)) {
        [UIPasteboard generalPasteboard].string = [self.tableView cellForRowAtIndexPath:indexPath].detailTextLabel.text ?: @"";
    }
}

#pragma mark - UIImagePickerControllerDelegate

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info {
    __weak typeof(self) weakSelf = self;
    [self.profile uploadImage:info[UIImagePickerControllerOriginalImage] withSession:self.manager.session completion:^(NSURL *imageURL, NSError *error){
        if (!error) {
            [weakSelf.manager reloadData];
        } else {
            [self handleFriendError:error];
        }
    }];
    [picker dismissViewControllerAnimated:YES completion:nil];
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    [picker dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - Internal

- (MRFriend *)profile {
    if (self.friend) {
        for (MRFriend *i in self.manager.friends) {
            if ([i.key isEqual:self.friend.key]) {
                return i;
            }
        }
        return nil;
    } else {
        return self.manager.profile;
    }
}

@end
