//
//  DismissAtZoomMRMapViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 7/3/19.
//  Copyright © 2019 Aruba Networks. All rights reserved.
//

import UIKit
import Meridian

private extension FloatingPoint {
    var degreesToRadians: Self { return self * .pi / 180 }
}

private extension Selector {
    static let mapControllerDoneButtonTapped = #selector(DismissAtZoomMRMapViewController.mapControllerDoneButtonTapped(_:))
}

class DismissAtZoomMRMapViewController: MRMapViewController {
    var initialNorthOffset: Double?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        navigationItem.leftBarButtonItem = UIBarButtonItem(barButtonSystemItem: .done, target: self, action: .mapControllerDoneButtonTapped)
    }
}

// MARK: - MRMapDelegate
extension DismissAtZoomMRMapViewController {
    override func mapView(_ mapView: MRMapView, visibleMapRectDidChange animated: Bool) {
        super.mapView(mapView, visibleMapRectDidChange: animated)
        
        // Minimum zoomscale required to fit the entire map on screen
        let minimumZoomScale = min(mapView.frame.size.width / mapView.map.size.width,
                                   mapView.frame.size.height / mapView.map.size.height)
        
        // Dismiss the meridian map if we zoom out much farther than that.
        if minimumZoomScale * 0.9 > mapView.zoomScale {
            dismiss(animated: true, completion: nil)
        }
        
    }
    
    override func mapViewDidFinishLoadingMap(_ mapView: MRMapView) {
        super.mapViewDidFinishLoadingMap(mapView)
        // Line up the Meridian map with the Apple map north
        if let initialNorthOffset = initialNorthOffset {
            let rotation = (360 - (mapView.map.northOffset + CGFloat(initialNorthOffset))).degreesToRadians
            mapView.setRotationAngle(rotation, animated: false, completion: nil)
        }
    }
}

extension DismissAtZoomMRMapViewController {
    @objc func mapControllerDoneButtonTapped(_ sender: UIBarButtonItem) {
        dismiss(animated: true, completion: nil)
    }
}

