//
//  MSCustomRouteViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/22/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSCustomRouteViewController: MRMapViewController, MSExampleViewController {
    private var routeOverlay: MRPathOverlay?
    
    convenience init() {
        self.init(nibName:nil, bundle:nil)
        mapView.mapKey = MREditorKey(forMap: MSExampleHost.mapID(), app: MSExampleHost.appID())
        mapView.showsUserLocation = true
        mapView.showsRoutePath = false
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Custom Route Line"
    }

    static func exampleDetail() -> String {
        return """
        Example of how to customize the route path.
        
        • The default route path is now a red line of width 2.
        """
    }

    // MARK: - MRMapViewDelegate

    override func mapView(_ mapView: MRMapView, rendererFor overlay: MRPathOverlay) -> MRPathRenderer? {
        let renderer = MRPathRenderer(overlay: overlay)
        renderer.strokeColor = UIColor.red
        renderer.lineCap = .round
        renderer.lineJoin = .round
        renderer.lineWidth = 2
        return renderer
    }

    override func mapViewDidFinishLoadingMap(_ mapView: MRMapView) {
        super.mapViewDidFinishLoadingMap(mapView)
        reloadRouteOverlay()
    }

    override func mapView(_ mapView: MRMapView, routeDidChange route: MRRoute) {
        super.mapView(mapView, routeDidChange: route)
        reloadRouteOverlay()
    }

    // MARK: - Internal

    private func reloadRouteOverlay() {
        if let overlay = routeOverlay {
            mapView.remove(overlay)
            routeOverlay = nil
        }
        if let route = mapView.route, let steps = route.steps {
            let path = UIBezierPath()
            for step in steps where step.mapKey == mapView.mapKey {
                path.append(step.path)
            }
            routeOverlay = MRPathOverlay(path: path.cgPath)
            mapView.add(routeOverlay!)
        }
    }
}
