//
//  MSCustomUserAnnotationViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/19/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSUserAnnotationView: MRAnnotationView {
    private lazy var shapeLayer: CAShapeLayer = {
        let shapeLayer = CAShapeLayer()
        shapeLayer.path = UIBezierPath.init(ovalIn: CGRect(x: 0, y: 0, width: 20, height: 20)).cgPath
        shapeLayer.lineWidth = 3
        shapeLayer.fillColor = UIColor.orange.cgColor
        shapeLayer.strokeColor = UIColor.white.cgColor
        shapeLayer.shadowOpacity = 0.5
        shapeLayer.shadowOffset = CGSize(width: 0, height: 1)
        return shapeLayer
    }()

    override init(annotation: MRAnnotation, reuseIdentifier: String) {
        super.init(annotation: annotation, reuseIdentifier: reuseIdentifier)
        initializeShapeLayer()
    }

    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        initializeShapeLayer()
    }

    override init(frame: CGRect) {
        super.init(frame: frame)
        initializeShapeLayer()
    }

    func initializeShapeLayer() {
        layer.addSublayer(shapeLayer)
        frame = CGRect(x: 0, y: 0, width: 20, height: 20)
    }
}

class MSCustomUserAnnotationViewController: MRMapViewController, MSExampleViewController {
    convenience init() {
        self.init(nibName:nil, bundle:nil)
        mapView.mapKey = MREditorKey(forMap: MSExampleHost.mapID(), app: MSExampleHost.appID())
        mapView.showsUserLocation = true
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Custom User Annotation"
    }

    static func exampleDetail() -> String {
        return """
        Example of how to customize annotations by returning a MRAnnotationView subclass.
        
        • Custom orange user location annotation.
        """
    }

    // MARK: - MRMapViewDelegate

    override func mapView(_ mapView: MRMapView, viewFor annotation: MRAnnotation?) -> MRAnnotationView? {
        let superAnnotationView = super.mapView(mapView, viewFor: annotation)

        guard let placemark = annotation as? MRUserLocation else {
            return superAnnotationView
        }
        if let view = mapView.dequeueReusableAnnotationView(withIdentifier: "MSUserAnnotationView") {
            return view
        }
        return MSUserAnnotationView.init(annotation: placemark, reuseIdentifier: "MSUserAnnotationView")
    }
}
