//
//  MSExamplesTableViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/17/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSExamplesTableViewController: UITableViewController {
    private var exampleData = [(sectionHeader: String, viewControllers: [MSExampleViewController.Type])]()
    private var selectedExample: MSExampleViewController.Type?

    override func viewDidLoad() {
        super.viewDidLoad()
        title = "Meridian SDK Examples"
        exampleData = [
            ("General", [
                MSMapViewController.self,
                MSDelegateViewController.self,
                MSConvertCoordinatesViewController.self,
                MSVisibleMapRectViewController.self,
                MSEmbeddedMapViewController.self
                ]),
            ("Apple Maps", [
                MSAppleMapViewController.self
            ]),
            ("Map UI", [
                MSLoadingViewController.self,
                MSShowsPlacemarksViewController.self,
                MSShowsButtonsViewController.self,
                MSShowsLocationViewController.self
                ]),
            ("Annotations and Overlays", [
                MSAddAnnotationViewController.self,
                MSSelectAnnotationViewController.self,
                MSCustomAnnotationViewController.self,
                MSCustomUserAnnotationViewController.self,
                MSOverlayViewController.self,
                MSCustomAnnotationSubclassViewController.self
                ]),
            ("Routes", [
                MSDirectionsViewController.self,
                MSCustomRouteViewController.self,
                MSManualRoutingViewController.self
                ]),
            ("Campaigns", [
                MSCampaignsTableViewController.self
                ]),
            ("Loading Placemarks", [
                MSPlacemarkListTableViewController.self,
                MSSearchTableViewController.self,
                MSNearbyTableViewController.self
                ]),
            ("Friends", [
                MSFriendTableViewController.self,
                MSFriendTableViewController2.self,
                MSFriendTableViewController3.self
                ]),
            ("Asset Tracking", [
                MSAssetTrackingMapViewController.self
                ])
        ]
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        selectedExample = nil
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return exampleData.count
    }

    override func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        if section < exampleData.count {
            return exampleData[section].sectionHeader
        }
        return nil
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if section < exampleData.count {
            return exampleData[section].viewControllers.count
        }
        return 0
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "exampleCell", for: indexPath)
        if let exampleProtocol = classForIndexPath(indexPath: indexPath) {
            cell.textLabel?.text = exampleProtocol.exampleTitle()
        }
        cell.accessoryType = .disclosureIndicator
        return cell
    }

    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        selectedExample = classForIndexPath(indexPath: indexPath)
        performSegue(withIdentifier: "ExampleDetailSegue", sender: self)
    }

    // MARK: - Navigation

    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if let detailController = segue.destination as? MSExampleDetailTableViewController {
            detailController.exampleController = selectedExample
        }
    }

    private func classForIndexPath(indexPath: IndexPath) -> MSExampleViewController.Type? {
        if indexPath.section < exampleData.count && indexPath.row < exampleData[indexPath.section].viewControllers.count {
            return exampleData[indexPath.section].viewControllers[indexPath.row]
        }
        return nil
    }
}
