//
//  MSSelectAnnotationViewController.swift
//  MeridianSwiftSamples
//
//  Created by Stephen Kelly on 5/19/17.
//  Copyright © 2017 Aruba Networks. All rights reserved.
//

import UIKit

class MSSelectAnnotationViewController: MRMapViewController, MSExampleViewController {
    private var selectedOnLoad = false
    
    convenience init() {
        self.init(nibName:nil, bundle:nil)
        mapView.mapKey = MREditorKey(forMap: MSExampleHost.mapID(), app: MSExampleHost.appID())
    }

    override func viewDidLoad() {
        super.viewDidLoad()

        let button0 = UIBarButtonItem(title: "Select", style: .plain, target: self, action: #selector(selectAnnotation))
        let button1 = UIBarButtonItem(title: "Deselect", style: .plain, target: self, action: #selector(deselectAnnotation))
        navigationItem.rightBarButtonItems = [button0, button1]
    }

    // MARK: - MSExampleViewController

    static func exampleTitle() -> String {
        return "Select Annotation"
    }

    static func exampleDetail() -> String {
        return """
        Example of how to programatically select annotations.
        
        • `Converse` annotation starts initially selected.
        • Button to select/deselect the `Gant` annotation.
        """
    }

    // MARK: - MRMapViewDelegate

    override func mapView(_ mapView: MRMapView, didAdd views: [MRAnnotationView]) {
        super.mapView(mapView, didAdd: views)
        if selectedOnLoad {
            return
        }
        for view in views {
            if let title = view.annotation.title, title == "Converse" {
                mapView.selectAnnotation(view.annotation, animated: true)
                selectedOnLoad = true
            }
        }
    }

    // MARK: - Action

    @objc private func selectAnnotation() {
        mapView.deselectAnnotation(animated: true)
        for placemark in mapView.placemarks {
            if let title = placemark.title, title == "Gant" {
                mapView.selectAnnotation(placemark, animated: true)
                break
            }
        }
    }

    @objc private func deselectAnnotation() {
        mapView.deselectAnnotation(animated: true)
    }
}
